@@ -53,36 +53,36 @@ int spa_v4l2_open(struct spa_v4l2_device *dev, const char *path)
 		return 0;
 
 	if (path == NULL) {
-		spa_log_error(dev->log, "v4l2: Device property not set");
+		spa_log_error(dev->log, "Device property not set");
 		return -EIO;
 	}
 
-	spa_log_info(dev->log, "v4l2: Playback device is '%s'", path);
+	spa_log_info(dev->log, "Playback device is '%s'", path);
 
 	dev->fd = open(path, O_RDWR | O_NONBLOCK, 0);
 	if (dev->fd == -1) {
 		err = errno;
-		spa_log_error(dev->log, "v4l2: Cannot open '%s': %d, %s",
+		spa_log_error(dev->log, "Cannot open '%s': %d, %s",
 			      path, err, strerror(err));
 		goto error;
 	}
 
 	if (fstat(dev->fd, &st) < 0) {
 		err = errno;
-		spa_log_error(dev->log, "v4l2: Cannot identify '%s': %d, %s",
+		spa_log_error(dev->log, "Cannot identify '%s': %d, %s",
 				path, err, strerror(err));
 		goto error_close;
 	}
 
 	if (!S_ISCHR(st.st_mode)) {
-		spa_log_error(dev->log, "v4l2: %s is no device", path);
+		spa_log_error(dev->log, "%s is no device", path);
 		err = ENODEV;
 		goto error_close;
 	}
 
 	if (xioctl(dev->fd, VIDIOC_QUERYCAP, &dev->cap) < 0) {
 		err = errno;
-		spa_log_error(dev->log, "v4l2: '%s' QUERYCAP: %m", path);
+		spa_log_error(dev->log, "'%s' QUERYCAP: %m", path);
 		goto error_close;
 	}
 	return 0;
@@ -110,7 +110,7 @@ int spa_v4l2_close(struct spa_v4l2_device *dev)
 	if (dev->active || dev->have_format)
 		return 0;
 
-	spa_log_info(dev->log, "v4l2: close");
+	spa_log_info(dev->log, "close");
 
 	if (close(dev->fd))
 		spa_log_warn(dev->log, "close: %m");
@@ -134,7 +134,7 @@ static int spa_v4l2_buffer_recycle(struct impl *this, uint32_t buffer_id)
 
 	if (xioctl(dev->fd, VIDIOC_QBUF, &b->v4l2_buffer) < 0) {
 		err = errno;
-		spa_log_error(this->log, "v4l2: '%s' VIDIOC_QBUF: %m", this->props.device);
+		spa_log_error(this->log, "'%s' VIDIOC_QBUF: %m", this->props.device);
 		return -err;
 	}
 
@@ -158,14 +158,14 @@ static int spa_v4l2_clear_buffers(struct impl *this)
 		d = b->outbuf->datas;
 
 		if (SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_OUTSTANDING)) {
-			spa_log_debug(this->log, "v4l2: queueing outstanding buffer %p", b);
+			spa_log_debug(this->log, "queueing outstanding buffer %p", b);
 			spa_v4l2_buffer_recycle(this, i);
 		}
 		if (SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_MAPPED)) {
 			munmap(b->ptr, d[0].maxsize);
 		}
 		if (SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_ALLOCATED)) {
-			spa_log_debug(this->log, "v4l2: close %d", (int) d[0].fd);
+			spa_log_debug(this->log, "close %d", (int) d[0].fd);
 			close(d[0].fd);
 		}
 		d[0].type = SPA_ID_INVALID;
@@ -590,12 +590,12 @@ spa_v4l2_enum_format(struct impl *this, int seq,
 			fmt.fmt.pix.height = 0;
 
 			if ((res = xioctl(dev->fd, VIDIOC_TRY_FMT, &fmt)) < 0) {
-				spa_log_debug(this->log, "v4l2: '%s' VIDIOC_TRY_FMT %08x: %m",
+				spa_log_debug(this->log, "'%s' VIDIOC_TRY_FMT %08x: %m",
 						this->props.device, info->fourcc);
 				goto next_fmtdesc;
 			}
 			if (fmt.fmt.pix.pixelformat != info->fourcc) {
-				spa_log_debug(this->log, "v4l2: '%s' VIDIOC_TRY_FMT wanted %.4s gave %.4s",
+				spa_log_debug(this->log, "'%s' VIDIOC_TRY_FMT wanted %.4s gave %.4s",
 						this->props.device, (char*)&info->fourcc,
 						(char*)&fmt.fmt.pix.pixelformat);
 				goto next_fmtdesc;
@@ -607,7 +607,7 @@ spa_v4l2_enum_format(struct impl *this, int seq,
 					goto enum_end;
 
 				res = -errno;
-				spa_log_error(this->log, "v4l2: '%s' VIDIOC_ENUM_FMT: %m",
+				spa_log_error(this->log, "'%s' VIDIOC_ENUM_FMT: %m",
 						this->props.device);
 				goto exit;
 			}
@@ -653,14 +653,16 @@ spa_v4l2_enum_format(struct impl *this, int seq,
 				goto next_fmtdesc;
 
 			res = -errno;
-			spa_log_error(this->log, "v4l2: '%s' VIDIOC_ENUM_FRAMESIZES: %m",
+			spa_log_error(this->log, "'%s' VIDIOC_ENUM_FRAMESIZES: %m",
 					this->props.device);
 			goto exit;
 		}
 		if (filter) {
+			static const struct spa_rectangle step = {1, 1};
+
+			const struct spa_rectangle *values;
 			const struct spa_pod_prop *p;
 			struct spa_pod *val;
-			const struct spa_rectangle step = { 1, 1 }, *values;
 			uint32_t choice, i, n_values;
 
 			/* check if we have a fixed frame size */
@@ -724,7 +726,35 @@ spa_v4l2_enum_format(struct impl *this, int seq,
 		spa_pod_builder_id(&b, info->format);
 	}
 	spa_pod_builder_prop(&b, SPA_FORMAT_VIDEO_size, 0);
-	spa_pod_builder_rectangle(&b, port->frmsize.discrete.width, port->frmsize.discrete.height);
+	if (port->frmsize.type == V4L2_FRMSIZE_TYPE_DISCRETE) {
+		spa_pod_builder_rectangle(&b,
+				port->frmsize.discrete.width,
+				port->frmsize.discrete.height);
+	} else if (port->frmsize.type == V4L2_FRMSIZE_TYPE_CONTINUOUS ||
+		   port->frmsize.type == V4L2_FRMSIZE_TYPE_STEPWISE) {
+		spa_pod_builder_push_choice(&b, &f[1], SPA_CHOICE_None, 0);
+		choice = (struct spa_pod_choice*)spa_pod_builder_frame(&b, &f[1]);
+
+		spa_pod_builder_rectangle(&b,
+				port->frmsize.stepwise.min_width,
+				port->frmsize.stepwise.min_height);
+		spa_pod_builder_rectangle(&b,
+				port->frmsize.stepwise.min_width,
+				port->frmsize.stepwise.min_height);
+		spa_pod_builder_rectangle(&b,
+				port->frmsize.stepwise.max_width,
+				port->frmsize.stepwise.max_height);
+
+		if (port->frmsize.type == V4L2_FRMSIZE_TYPE_CONTINUOUS) {
+			choice->body.type = SPA_CHOICE_Range;
+		} else {
+			choice->body.type = SPA_CHOICE_Step;
+			spa_pod_builder_rectangle(&b,
+					port->frmsize.stepwise.max_width,
+					port->frmsize.stepwise.max_height);
+		}
+		spa_pod_builder_pop(&b, &f[1]);
+	}
 
 	spa_pod_builder_prop(&b, SPA_FORMAT_VIDEO_framerate, 0);
 
@@ -744,15 +774,17 @@ spa_v4l2_enum_format(struct impl *this, int seq,
 					goto next_frmsize;
 				break;
 			}
-			spa_log_error(this->log, "v4l2: '%s' VIDIOC_ENUM_FRAMEINTERVALS: %m",
+			spa_log_error(this->log, "'%s' VIDIOC_ENUM_FRAMEINTERVALS: %m",
 					this->props.device);
 			goto exit;
 		}
 		if (filter) {
+			static const struct spa_fraction step = {1, 1};
+
+			const struct spa_fraction *values;
 			const struct spa_pod_prop *p;
 			struct spa_pod *val;
 			uint32_t i, n_values, choice;
-			const struct spa_fraction step = { 1, 1 }, *values;
 
 			if (!(p = spa_pod_find_prop(filter, NULL, SPA_FORMAT_VIDEO_framerate)))
 				goto have_framerate;
@@ -892,7 +924,7 @@ static int spa_v4l2_set_format(struct impl *this, struct spa_video_info *format,
 	info = find_format_info_by_media_type(format->media_type,
 					      format->media_subtype, video_format, 0);
 	if (info == NULL || size == NULL || framerate == NULL) {
-		spa_log_error(this->log, "v4l2: unknown media type %d %d %d", format->media_type,
+		spa_log_error(this->log, "unknown media type %d %d %d", format->media_type,
 			      format->media_subtype, video_format);
 		return -EINVAL;
 	}
@@ -905,7 +937,7 @@ static int spa_v4l2_set_format(struct impl *this, struct spa_video_info *format,
 	streamparm.parm.capture.timeperframe.numerator = framerate->denom;
 	streamparm.parm.capture.timeperframe.denominator = framerate->num;
 
-	spa_log_debug(this->log, "v4l2: set %.4s %dx%d %d/%d", (char *)&fmt.fmt.pix.pixelformat,
+	spa_log_debug(this->log, "set %.4s %dx%d %d/%d", (char *)&fmt.fmt.pix.pixelformat,
 		     fmt.fmt.pix.width, fmt.fmt.pix.height,
 		     streamparm.parm.capture.timeperframe.denominator,
 		     streamparm.parm.capture.timeperframe.numerator);
@@ -918,7 +950,7 @@ static int spa_v4l2_set_format(struct impl *this, struct spa_video_info *format,
 	cmd = (flags & SPA_NODE_PARAM_FLAG_TEST_ONLY) ? VIDIOC_TRY_FMT : VIDIOC_S_FMT;
 	if (xioctl(dev->fd, cmd, &fmt) < 0) {
 		res = -errno;
-		spa_log_error(this->log, "v4l2: '%s' VIDIOC_S_FMT: %m",
+		spa_log_error(this->log, "'%s' VIDIOC_S_FMT: %m",
 				this->props.device);
 		return res;
 	}
@@ -932,7 +964,7 @@ static int spa_v4l2_set_format(struct impl *this, struct spa_video_info *format,
 			reqfmt.fmt.pix.height == fmt.fmt.pix.height);
 
 	if (!match && !SPA_FLAG_IS_SET(flags, SPA_NODE_PARAM_FLAG_NEAREST)) {
-		spa_log_error(this->log, "v4l2: wanted %.4s %dx%d, got %.4s %dx%d",
+		spa_log_error(this->log, "wanted %.4s %dx%d, got %.4s %dx%d",
 				(char *)&reqfmt.fmt.pix.pixelformat,
 				reqfmt.fmt.pix.width, reqfmt.fmt.pix.height,
 				(char *)&fmt.fmt.pix.pixelformat,
@@ -943,7 +975,7 @@ static int spa_v4l2_set_format(struct impl *this, struct spa_video_info *format,
 	if (flags & SPA_NODE_PARAM_FLAG_TEST_ONLY)
 		return match ? 0 : 1;
 
-	spa_log_info(this->log, "v4l2: '%s' got %.4s %dx%d %d/%d",
+	spa_log_info(this->log, "'%s' got %.4s %dx%d %d/%d",
 			this->props.device, (char *)&fmt.fmt.pix.pixelformat,
 			fmt.fmt.pix.width, fmt.fmt.pix.height,
 			streamparm.parm.capture.timeperframe.denominator,
@@ -1090,7 +1122,7 @@ spa_v4l2_enum_controls(struct impl *this, int seq,
 			goto next;
 		}
 		res = -errno;
-		spa_log_error(this->log, "v4l2: '%s' VIDIOC_QUERYCTRL: %m", this->props.device);
+		spa_log_error(this->log, "'%s' VIDIOC_QUERYCTRL: %m", this->props.device);
 		return res;
 	}
 	if (result.next & next_fl)
@@ -1128,14 +1160,14 @@ spa_v4l2_enum_controls(struct impl *this, int seq,
 							queryctrl.minimum,
 							queryctrl.maximum,
 							queryctrl.step),
-			SPA_PROP_INFO_name, SPA_POD_String(queryctrl.name));
+			SPA_PROP_INFO_description, SPA_POD_String(queryctrl.name));
 		break;
 	case V4L2_CTRL_TYPE_BOOLEAN:
 		param = spa_pod_builder_add_object(&b,
 			SPA_TYPE_OBJECT_PropInfo, SPA_PARAM_PropInfo,
 			SPA_PROP_INFO_id,   SPA_POD_Id(prop_id),
 			SPA_PROP_INFO_type, SPA_POD_CHOICE_Bool(queryctrl.default_value),
-			SPA_PROP_INFO_name, SPA_POD_String(queryctrl.name));
+			SPA_PROP_INFO_description, SPA_POD_String(queryctrl.name));
 		break;
 	case V4L2_CTRL_TYPE_MENU:
 	{
@@ -1146,7 +1178,7 @@ spa_v4l2_enum_controls(struct impl *this, int seq,
 		spa_pod_builder_add(&b,
 			SPA_PROP_INFO_id,    SPA_POD_Id(prop_id),
 			SPA_PROP_INFO_type,  SPA_POD_CHOICE_ENUM_Int(1, queryctrl.default_value),
-			SPA_PROP_INFO_name,  SPA_POD_String(queryctrl.name),
+			SPA_PROP_INFO_description,  SPA_POD_String(queryctrl.name),
 			0);
 
 		spa_zero(querymenu);
@@ -1257,7 +1289,7 @@ static void v4l2_on_fd_events(struct spa_source *source)
 
 	if (source->rmask & SPA_IO_ERR) {
 		struct port *port = &this->out_ports[0];
-		spa_log_error(this->log, "v4l2: '%p' error %08x", this->props.device, source->rmask);
+		spa_log_error(this->log, "'%p' error %08x", this->props.device, source->rmask);
 		if (port->source.loop)
 			spa_loop_remove_source(this->data_loop, &port->source);
 		return;
@@ -1307,7 +1339,7 @@ static int spa_v4l2_use_buffers(struct impl *this, struct spa_buffer **buffers,
 		} else if (d[0].type == SPA_DATA_DmaBuf) {
 			port->memtype = V4L2_MEMORY_DMABUF;
 		} else {
-			spa_log_error(this->log, "v4l2: can't use buffers of type %d", d[0].type);
+			spa_log_error(this->log, "can't use buffers of type %d", d[0].type);
 			return -EINVAL;
 		}
 	}
@@ -1318,12 +1350,12 @@ static int spa_v4l2_use_buffers(struct impl *this, struct spa_buffer **buffers,
 	reqbuf.count = n_buffers;
 
 	if (xioctl(dev->fd, VIDIOC_REQBUFS, &reqbuf) < 0) {
-		spa_log_error(this->log, "v4l2: '%s' VIDIOC_REQBUFS %m", this->props.device);
+		spa_log_error(this->log, "'%s' VIDIOC_REQBUFS %m", this->props.device);
 		return -errno;
 	}
-	spa_log_debug(this->log, "v4l2: got %d buffers", reqbuf.count);
+	spa_log_debug(this->log, "got %d buffers", reqbuf.count);
 	if (reqbuf.count < n_buffers) {
-		spa_log_error(this->log, "v4l2: '%s' can't allocate enough buffers %d < %d",
+		spa_log_error(this->log, "'%s' can't allocate enough buffers %d < %d",
 				this->props.device, reqbuf.count, n_buffers);
 		return -ENOMEM;
 	}
@@ -1337,10 +1369,10 @@ static int spa_v4l2_use_buffers(struct impl *this, struct spa_buffer **buffers,
 		b->flags = BUFFER_FLAG_OUTSTANDING;
 		b->h = spa_buffer_find_meta_data(buffers[i], SPA_META_Header, sizeof(*b->h));
 
-		spa_log_debug(this->log, "v4l2: import buffer %p", buffers[i]);
+		spa_log_debug(this->log, "import buffer %p", buffers[i]);
 
 		if (buffers[i]->n_datas < 1) {
-			spa_log_error(this->log, "v4l2: invalid memory on buffer %p", buffers[i]);
+			spa_log_error(this->log, "invalid memory on buffer %p", buffers[i]);
 			return -EINVAL;
 		}
 		d = buffers[i]->datas;
@@ -1375,7 +1407,7 @@ static int spa_v4l2_use_buffers(struct impl *this, struct spa_buffer **buffers,
 			b->v4l2_buffer.m.fd = d[0].fd;
 		}
 		else {
-			spa_log_error(this->log, "v4l2: invalid port memory %d",
+			spa_log_error(this->log, "invalid port memory %d",
 					port->memtype);
 			return -EIO;
 		}
@@ -1405,15 +1437,15 @@ mmap_init(struct impl *this,
 	reqbuf.count = n_buffers;
 
 	if (xioctl(dev->fd, VIDIOC_REQBUFS, &reqbuf) < 0) {
-		spa_log_error(this->log, "v4l2: '%s' VIDIOC_REQBUFS: %m", this->props.device);
+		spa_log_error(this->log, "'%s' VIDIOC_REQBUFS: %m", this->props.device);
 		return -errno;
 	}
 
-	spa_log_debug(this->log, "v4l2: got %d buffers", reqbuf.count);
+	spa_log_debug(this->log, "got %d buffers", reqbuf.count);
 	n_buffers = reqbuf.count;
 
 	if (n_buffers < 2) {
-		spa_log_error(this->log, "v4l2: '%s' can't allocate enough buffers (%d)",
+		spa_log_error(this->log, "'%s' can't allocate enough buffers (%d)",
 				this->props.device, n_buffers);
 		return -ENOMEM;
 	}
@@ -1423,7 +1455,7 @@ mmap_init(struct impl *this,
 		struct spa_data *d;
 
 		if (buffers[i]->n_datas < 1) {
-			spa_log_error(this->log, "v4l2: invalid buffer data");
+			spa_log_error(this->log, "invalid buffer data");
 			return -EINVAL;
 		}
 
@@ -1439,13 +1471,13 @@ mmap_init(struct impl *this,
 		b->v4l2_buffer.index = i;
 
 		if (xioctl(dev->fd, VIDIOC_QUERYBUF, &b->v4l2_buffer) < 0) {
-			spa_log_error(this->log, "v4l2: '%s' VIDIOC_QUERYBUF: %m", this->props.device);
+			spa_log_error(this->log, "'%s' VIDIOC_QUERYBUF: %m", this->props.device);
 			return -errno;
 		}
 
 		if (b->v4l2_buffer.flags & V4L2_BUF_FLAG_QUEUED) {
 			/* some drivers can give us an already queued buffer. */
-			spa_log_warn(this->log, "v4l2: buffer %d was already queued", i);
+			spa_log_warn(this->log, "buffer %d was already queued", i);
 			n_buffers = i;
 			break;
 		}
@@ -1458,11 +1490,11 @@ mmap_init(struct impl *this,
 		d[0].chunk->stride = port->fmt.fmt.pix.bytesperline;
 		d[0].chunk->flags = 0;
 
-		spa_log_debug(this->log, "v4l2: data types %08x", d[0].type);
+		spa_log_debug(this->log, "data types %08x", d[0].type);
 
 		if (port->have_expbuf &&
 		    d[0].type != SPA_ID_INVALID &&
-		    (d[0].type & (1u << SPA_DATA_DmaBuf))) {
+		    (d[0].type & ((1u << SPA_DATA_DmaBuf)|(1u<<SPA_DATA_MemFd)))) {
 			struct v4l2_exportbuffer expbuf;
 
 			spa_zero(expbuf);
@@ -1471,20 +1503,23 @@ mmap_init(struct impl *this,
 			expbuf.flags = O_CLOEXEC | O_RDONLY;
 			if (xioctl(dev->fd, VIDIOC_EXPBUF, &expbuf) < 0) {
 				if (errno == ENOTTY || errno == EINVAL) {
-					spa_log_debug(this->log, "v4l2: '%s' VIDIOC_EXPBUF not supported: %m",
+					spa_log_debug(this->log, "'%s' VIDIOC_EXPBUF not supported: %m",
 							this->props.device);
 					port->have_expbuf = false;
 					goto fallback;
 				}
-				spa_log_error(this->log, "v4l2: '%s' VIDIOC_EXPBUF: %m", this->props.device);
+				spa_log_error(this->log, "'%s' VIDIOC_EXPBUF: %m", this->props.device);
 				return -errno;
 			}
-			d[0].type = SPA_DATA_DmaBuf;
+			if (d[0].type & (1u<<SPA_DATA_DmaBuf))
+				d[0].type = SPA_DATA_DmaBuf;
+			else
+				d[0].type = SPA_DATA_MemFd;
 			d[0].flags = SPA_DATA_FLAG_READABLE;
 			d[0].fd = expbuf.fd;
 			d[0].data = NULL;
 			SPA_FLAG_SET(b->flags, BUFFER_FLAG_ALLOCATED);
-			spa_log_debug(this->log, "v4l2: EXPBUF fd:%d", expbuf.fd);
+			spa_log_debug(this->log, "EXPBUF fd:%d", expbuf.fd);
 			use_expbuf = true;
 		} else if (d[0].type & (1u << SPA_DATA_MemPtr)) {
 fallback:
@@ -1498,20 +1533,20 @@ fallback:
 					dev->fd,
 					b->v4l2_buffer.m.offset);
 			if (d[0].data == MAP_FAILED) {
-				spa_log_error(this->log, "v4l2: '%s' mmap: %m", this->props.device);
+				spa_log_error(this->log, "'%s' mmap: %m", this->props.device);
 				return -errno;
 			}
 			b->ptr = d[0].data;
 			SPA_FLAG_SET(b->flags, BUFFER_FLAG_MAPPED);
-			spa_log_debug(this->log, "v4l2: mmap offset:%u data:%p", d[0].mapoffset, b->ptr);
+			spa_log_debug(this->log, "mmap offset:%u data:%p", d[0].mapoffset, b->ptr);
 			use_expbuf = false;
 		} else {
-			spa_log_error(this->log, "v4l2: unsupported data type:%08x", d[0].type);
+			spa_log_error(this->log, "unsupported data type:%08x", d[0].type);
 			return -ENOTSUP;
 		}
 		spa_v4l2_buffer_recycle(this, i);
 	}
-	spa_log_info(this->log, "v4l2: have %u buffers using %s", n_buffers,
+	spa_log_info(this->log, "have %u buffers using %s", n_buffers,
 			use_expbuf ? "EXPBUF" : "MMAP");
 
 	port->n_buffers = n_buffers;
@@ -1549,7 +1584,7 @@ spa_v4l2_alloc_buffers(struct impl *this,
 		if ((res = read_init(this)) < 0)
 			return res;
 	} else {
-		spa_log_error(this->log, "v4l2: invalid capabilities %08x",
+		spa_log_error(this->log, "invalid capabilities %08x",
 					dev->cap.capabilities);
 		return -EIO;
 	}
@@ -1576,7 +1611,7 @@ static int spa_v4l2_stream_on(struct impl *this)
 
 	type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 	if (xioctl(dev->fd, VIDIOC_STREAMON, &type) < 0) {
-		spa_log_error(this->log, "v4l2: '%s' VIDIOC_STREAMON: %m", this->props.device);
+		spa_log_error(this->log, "'%s' VIDIOC_STREAMON: %m", this->props.device);
 		return -errno;
 	}
 
@@ -1624,7 +1659,7 @@ static int spa_v4l2_stream_off(struct impl *this)
 
 	type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 	if (xioctl(dev->fd, VIDIOC_STREAMOFF, &type) < 0) {
-		spa_log_error(this->log, "v4l2: '%s' VIDIOC_STREAMOFF: %m", this->props.device);
+		spa_log_error(this->log, "'%s' VIDIOC_STREAMOFF: %m", this->props.device);
 		return -errno;
 	}
 	for (i = 0; i < port->n_buffers; i++) {
