@@ -47,18 +47,22 @@ struct buffer_attr {
 	uint32_t fragsize;
 };
 
+enum stream_type {
+	STREAM_TYPE_RECORD,
+	STREAM_TYPE_PLAYBACK,
+	STREAM_TYPE_UPLOAD,
+};
+
 struct stream {
 	struct spa_list link;
 	uint32_t create_tag;
 	uint32_t channel;	/* index in map */
 	uint32_t id;		/* id of global */
+	uint32_t index;		/* index */
 
 	struct impl *impl;
 	struct client *client;
-#define STREAM_TYPE_RECORD	0
-#define STREAM_TYPE_PLAYBACK	1
-#define STREAM_TYPE_UPLOAD	2
-	uint32_t type;
+	enum stream_type type;
 	enum pw_direction direction;
 
 	struct pw_properties *props;
@@ -66,7 +70,6 @@ struct stream {
 	struct pw_stream *stream;
 	struct spa_hook stream_listener;
 
-	struct spa_io_rate_match *rate_match;
 	struct spa_io_position *position;
 	struct spa_ringbuffer ring;
 	void *buffer;
@@ -80,9 +83,15 @@ struct stream {
 	int64_t delay;
 
 	uint32_t last_quantum;
-	int64_t missing;
 	int64_t requested;
 
+	struct spa_fraction min_req;
+	struct spa_fraction default_req;
+	struct spa_fraction min_frag;
+	struct spa_fraction default_frag;
+	struct spa_fraction default_tlength;
+	struct spa_fraction min_quantum;
+
 	struct sample_spec ss;
 	struct channel_map map;
 	struct buffer_attr attr;
@@ -101,15 +110,18 @@ struct stream {
 	unsigned int adjust_latency:1;
 	unsigned int is_underrun:1;
 	unsigned int in_prebuf:1;
-	unsigned int done:1;
 	unsigned int killed:1;
+	unsigned int pending:1;
 };
 
+struct stream *stream_new(struct client *client, enum stream_type type, uint32_t create_tag,
+			  const struct sample_spec *ss, const struct channel_map *map,
+			  const struct buffer_attr *attr);
 void stream_free(struct stream *stream);
 void stream_flush(struct stream *stream);
 uint32_t stream_pop_missing(struct stream *stream);
 
-int stream_send_underflow(struct stream *stream, int64_t offset, uint32_t underrun_for);
+int stream_send_underflow(struct stream *stream, int64_t offset);
 int stream_send_overflow(struct stream *stream);
 int stream_send_killed(struct stream *stream);
 int stream_send_started(struct stream *stream);
