@@ -29,8 +29,10 @@
 #include "pipewire/log.h"
 #include "pipewire/data-loop.h"
 #include "pipewire/private.h"
+#include "pipewire/thread.h"
 
-#define NAME "data-loop"
+PW_LOG_TOPIC_EXTERN(log_data_loop);
+#define PW_LOG_TOPIC_DEFAULT log_data_loop
 
 SPA_EXPORT
 int pw_data_loop_wait(struct pw_data_loop *this, int timeout)
@@ -60,7 +62,7 @@ void pw_data_loop_exit(struct pw_data_loop *this)
 static void thread_cleanup(void *arg)
 {
 	struct pw_data_loop *this = arg;
-	pw_log_debug(NAME" %p: leave thread", this);
+	pw_log_debug("%p: leave thread", this);
 	this->running = false;
 	pw_loop_leave(this->loop);
 }
@@ -70,7 +72,7 @@ static void *do_loop(void *user_data)
 	struct pw_data_loop *this = user_data;
 	int res;
 
-	pw_log_debug(NAME" %p: enter thread", this);
+	pw_log_debug("%p: enter thread", this);
 	pw_loop_enter(this->loop);
 
 	pthread_cleanup_push(thread_cleanup, this);
@@ -79,7 +81,7 @@ static void *do_loop(void *user_data)
 		if ((res = pw_loop_iterate(this->loop, -1)) < 0) {
 			if (res == -EINTR)
 				continue;
-			pw_log_error(NAME" %p: iterate error %d (%s)",
+			pw_log_error("%p: iterate error %d (%s)",
 					this, res, spa_strerror(res));
 		}
 	}
@@ -88,11 +90,13 @@ static void *do_loop(void *user_data)
 	return NULL;
 }
 
-static void do_stop(void *data, uint64_t count)
+static int do_stop(struct spa_loop *loop, bool async, uint32_t seq,
+		const void *data, size_t size, void *user_data)
 {
-	struct pw_data_loop *this = data;
-	pw_log_debug(NAME" %p: stopping", this);
+	struct pw_data_loop *this = user_data;
+	pw_log_debug("%p: stopping", this);
 	this->running = false;
+	return 0;
 }
 
 static struct pw_data_loop *loop_new(struct pw_loop *loop, const struct spa_dict *props)
@@ -107,7 +111,7 @@ static struct pw_data_loop *loop_new(struct pw_loop *loop, const struct spa_dict
 		goto error_cleanup;
 	}
 
-	pw_log_debug(NAME" %p: new", this);
+	pw_log_debug("%p: new", this);
 
 	if (loop == NULL) {
 		loop = pw_loop_new(props);
@@ -115,28 +119,19 @@ static struct pw_data_loop *loop_new(struct pw_loop *loop, const struct spa_dict
 	}
 	if (loop == NULL) {
 		res = -errno;
-		pw_log_error(NAME" %p: can't create loop: %m", this);
+		pw_log_error("%p: can't create loop: %m", this);
 		goto error_free;
 	}
 	this->loop = loop;
 
-	if (props == NULL ||
-	    (str = spa_dict_lookup(props, "loop.cancel")) == NULL ||
-	    pw_properties_parse_bool(str) == false) {
-		this->event = pw_loop_add_event(this->loop, do_stop, this);
-		if (this->event == NULL) {
-			res = -errno;
-			pw_log_error(NAME" %p: can't add event: %m", this);
-			goto error_loop_destroy;
-		}
-	}
+	if (props != NULL &&
+	    (str = spa_dict_lookup(props, "loop.cancel")) != NULL)
+		this->cancel = pw_properties_parse_bool(str);
+
 	spa_hook_list_init(&this->listener_list);
 
 	return this;
 
-error_loop_destroy:
-	if (this->created && this->loop)
-		pw_loop_destroy(this->loop);
 error_free:
 	free(this);
 error_cleanup:
@@ -147,7 +142,6 @@ error_cleanup:
 /** Create a new \ref pw_data_loop.
  * \return a newly allocated data loop
  *
- * \memberof pw_data_loop
  */
 SPA_EXPORT
 struct pw_data_loop *pw_data_loop_new(const struct spa_dict *props)
@@ -158,19 +152,16 @@ struct pw_data_loop *pw_data_loop_new(const struct spa_dict *props)
 
 /** Destroy a data loop
  * \param loop the data loop to destroy
- * \memberof pw_data_loop
  */
 SPA_EXPORT
 void pw_data_loop_destroy(struct pw_data_loop *loop)
 {
-	pw_log_debug(NAME" %p: destroy", loop);
+	pw_log_debug("%p: destroy", loop);
 
 	pw_data_loop_emit_destroy(loop);
 
 	pw_data_loop_stop(loop);
 
-	if (loop->event)
-		pw_loop_destroy_source(loop->loop, loop->event);
 	if (loop->created)
 		pw_loop_destroy(loop->loop);
 
@@ -188,6 +179,7 @@ void pw_data_loop_add_listener(struct pw_data_loop *loop,
 	spa_hook_list_append(&loop->listener_list, listener, events, data);
 }
 
+SPA_EXPORT
 struct pw_loop *
 pw_data_loop_get_loop(struct pw_data_loop *loop)
 {
@@ -200,20 +192,26 @@ pw_data_loop_get_loop(struct pw_data_loop *loop)
  *
  * This will start the realtime thread that manages the loop.
  *
- * \memberof pw_data_loop
  */
 SPA_EXPORT
 int pw_data_loop_start(struct pw_data_loop *loop)
 {
 	if (!loop->running) {
-		int err;
+		struct spa_thread_utils *utils;
+		struct spa_thread *thr;
 
 		loop->running = true;
-		if ((err = pthread_create(&loop->thread, NULL, do_loop, loop)) != 0) {
-			pw_log_error(NAME" %p: can't create thread: %s", loop, strerror(err));
+
+		if ((utils = loop->thread_utils) == NULL)
+			utils = pw_thread_utils_get();
+		thr = spa_thread_utils_create(utils, NULL, do_loop, loop);
+		loop->thread = (pthread_t)thr;
+		if (thr == NULL) {
+			pw_log_error("%p: can't create thread: %m", loop);
 			loop->running = false;
-			return -err;
+			return -errno;
 		}
+		spa_thread_utils_acquire_rt(utils, thr, -1);
 	}
 	return 0;
 }
@@ -224,25 +222,27 @@ int pw_data_loop_start(struct pw_data_loop *loop)
  *
  * This will stop and join the realtime thread that manages the loop.
  *
- * \memberof pw_data_loop
  */
 SPA_EXPORT
 int pw_data_loop_stop(struct pw_data_loop *loop)
 {
-	pw_log_debug(NAME": %p stopping", loop);
+	pw_log_debug("%p stopping", loop);
 	if (loop->running) {
-		if (loop->event) {
-			pw_log_debug(NAME": %p signal", loop);
-			pw_loop_signal_event(loop->loop, loop->event);
-		} else {
-			pw_log_debug(NAME": %p cancel", loop);
+		struct spa_thread_utils *utils;
+		if (loop->cancel) {
+			pw_log_debug("%p cancel", loop);
 			pthread_cancel(loop->thread);
+		} else {
+			pw_log_debug("%p signal", loop);
+			pw_loop_invoke(loop->loop, do_stop, 1, NULL, 0, false, loop);
 		}
-		pw_log_debug(NAME": %p join", loop);
-		pthread_join(loop->thread, NULL);
-		pw_log_debug(NAME": %p joined", loop);
+		pw_log_debug("%p join", loop);
+		if ((utils = loop->thread_utils) == NULL)
+			utils = pw_thread_utils_get();
+		spa_thread_utils_join(utils, (struct spa_thread*)loop->thread, NULL);
+		pw_log_debug("%p joined", loop);
 	}
-	pw_log_debug(NAME": %p stopped", loop);
+	pw_log_debug("%p stopped", loop);
 	return 0;
 }
 
@@ -250,12 +250,23 @@ int pw_data_loop_stop(struct pw_data_loop *loop)
  * \param loop the data loop to check
  * \return true is the current thread is the data loop thread
  *
- * \memberof pw_data_loop
  */
 SPA_EXPORT
 bool pw_data_loop_in_thread(struct pw_data_loop * loop)
 {
-	return pthread_equal(loop->thread, pthread_self());
+	return loop->running && pthread_equal(loop->thread, pthread_self());
+}
+
+/** Get the thread object.
+ * \param loop the data loop to get the thread of
+ * \return the thread object or NULL when the thread is not running
+ *
+ * On posix based systems this returns a pthread_t
+ */
+SPA_EXPORT
+struct spa_thread *pw_data_loop_get_thread(struct pw_data_loop * loop)
+{
+	return loop->running ? (struct spa_thread*)loop->thread : NULL;
 }
 
 SPA_EXPORT
@@ -270,3 +281,17 @@ int pw_data_loop_invoke(struct pw_data_loop *loop,
 		res = func(loop->loop->loop, false, seq, data, size, user_data);
 	return res;
 }
+
+/** Set a thread utils implementation.
+ * \param loop the data loop to set the thread utils on
+ * \param impl the thread utils implementation
+ *
+ * This configures a custom spa_thread_utils implementation for this data
+ * loop. Use NULL to restore the system default implementation.
+ */
+SPA_EXPORT
+void pw_data_loop_set_thread_utils(struct pw_data_loop *loop,
+		struct spa_thread_utils *impl)
+{
+	loop->thread_utils = impl;
+}
