@@ -700,6 +700,7 @@ struct pw_impl_node {
 	unsigned int lock_rate:1;	/**< don't change graph rate */
 	unsigned int transport_sync:1;	/**< supports transport sync */
 	unsigned int current_pending:1;	/**< a quantum/rate update is pending */
+	unsigned int moved:1;		/**< the node was moved drivers */
 
 	uint32_t port_user_data_size;	/**< extra size for port user data */
 
@@ -1289,6 +1290,8 @@ void pw_settings_clean(struct pw_context *context);
 
 void pw_impl_module_schedule_destroy(struct pw_impl_module *module);
 
+pthread_attr_t *pw_thread_fill_attr(const struct spa_dict *props, pthread_attr_t *attr);
+
 /** \endcond */
 
 #ifdef __cplusplus
